@@ -1,14 +1,7 @@
 from __future__ import print_function
 
 import numpy as np
-from MEAutility.core import MEA, RectMEA, rotation_matrix
-import matplotlib.pylab as plt
-
-import matplotlib.patches as patches
-from matplotlib.path import Path
-from mpl_toolkits.mplot3d import art3d
-from matplotlib import colors as mpl_colors
-from mpl_toolkits.mplot3d.art3d import Poly3DCollection
+from MEAutility.core import rotation_matrix
 
 
 def plot_probe(mea, ax=None, xlim=None, ylim=None, color_currents=False, top=None, bottom=None,
@@ -47,6 +40,9 @@ def plot_probe(mea, ax=None, xlim=None, ylim=None, color_currents=False, top=Non
         The output axis
 
     '''
+    import matplotlib.pylab as plt
+    import matplotlib.patches as patches
+    from matplotlib.path import Path
 
     if ax is None:
         fig = plt.figure()
@@ -206,6 +202,7 @@ def plot_probe_3d(mea, ax=None, xlim=None, ylim=None, zlim=None, top=None, botto
         The output axis
 
     '''
+    import matplotlib.pylab as plt
     from matplotlib import colors as mpl_colors
     from mpl_toolkits.mplot3d.art3d import Poly3DCollection
 
@@ -355,6 +352,8 @@ def plot_v_image(mea, v_plane=None, x_bound=None, y_bound=None, z_bound=None, of
         The voltage image 
 
     '''
+    import matplotlib.pylab as plt
+
     if v_plane is None:
         v_grid = np.zeros((npoints, npoints))
         if plane == 'xy':
@@ -444,6 +443,8 @@ def plot_v_surf(mea, v_plane=None, x_bound=None, y_bound=None, z_bound=None, off
     v_grid: np.array
         The voltage image 
     '''
+    import matplotlib.pylab as plt
+
     if v_plane is None:
         v_grid = np.zeros((npoints, npoints))
         if plane == 'xy':
@@ -520,9 +521,9 @@ def plot_v_surf(mea, v_plane=None, x_bound=None, y_bound=None, z_bound=None, off
     return ax, v_grid.T
 
 
-def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
+def plot_mea_recording(signals, mea, colors=None, ax=None, spacing=None,
                        scalebar=False, time=None, vscale=None, hide_axis=True,
-                       axis_equal=False):
+                       axis_equal=False, lw=1, alpha=1, channels=None, **ax_kwargs):
     '''
     Plots mea signals at electrode locations.
 
@@ -557,6 +558,8 @@ def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
         The output axis
 
     '''
+    import matplotlib.pylab as plt
+
     if ax is None:
         fig = plt.figure()
         ax = fig.add_subplot(1, 1, 1, frameon=False)
@@ -566,7 +569,25 @@ def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
         no_tight = True
 
     mea_pos = np.array([np.dot(mea.positions, mea.main_axes[0]), np.dot(mea.positions, mea.main_axes[1])]).T
-    mea_pitch = [np.max(np.diff(np.sort(mea_pos[:, 0]))), np.max(np.diff(np.sort(mea_pos[:, 1])))]
+
+    if len(mea_pos) > 1:
+        diff_x = np.diff(mea_pos[:, 0])
+        diff_y = np.diff(mea_pos[:, 1])
+
+        x_pitch = 0
+        y_pitch = 0
+        if np.any(diff_x > 0):
+            x_pitch = np.min(diff_x[diff_x > 0])
+        if np.any(diff_y > 0):
+            y_pitch = np.min(diff_y[diff_y > 0])
+        if x_pitch == 0:
+            x_pitch = y_pitch
+        if y_pitch == 0:
+            y_pitch = x_pitch
+        mea_pitch = [x_pitch, y_pitch]
+    else:
+        # single channel
+        mea_pitch = [1, 1]
 
     if spacing is None:
         spacing = 0.1 * np.min(mea_pitch)
@@ -578,7 +599,7 @@ def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
 
     # normalize to min peak
     if vscale is None:
-        signalmin = 1.5 * np.max(np.abs(signals))
+        signalmin = 1.5 * np.nanmax(np.abs(signals))
         signals_norm = signals / signalmin * mea_pitch[1]
     else:
         signals_norm = signals / vscale * mea_pitch[1]
@@ -587,27 +608,42 @@ def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
         if len(signals.shape) > 2:
             colors = plt.rcParams['axes.prop_cycle'].by_key()['color']
         else:
-            colors = 'k'
+            colors = ['k']
+
+    # if color is passed as tuple
+    if isinstance(colors, tuple):
+        colors = [colors]
+    if isinstance(colors, str):
+        colors = [colors]
+
+    if channels is None:
+        channels = np.arange(mea.number_electrodes)
 
     number_electrodes = mea.number_electrodes
     if len(signals.shape) == 3:  # multiple
         for sp_i, sp in enumerate(signals_norm):
-            for el in range(number_electrodes):
+            for el in channels:
                 if len(colors) > 1:
-                    ax.plot(np.linspace(-(mea_pitch[0] - spacing) / 2., (mea_pitch[0] - spacing) / 2., signals.shape[2])
-                            + mea_pos[el, 0], np.transpose(sp[el, :]) + mea_pos[el, 1], lw=lw,
+                    ax.plot(np.linspace(-(mea_pitch[0] - spacing) / 2., (mea_pitch[0] - spacing) / 2.,
+                                        signals.shape[2])
+                            + mea_pos[el, 0], np.transpose(sp[el, :]) + mea_pos[el, 1],
                             color=colors[int(np.mod(sp_i, len(colors)))],
-                            label='EAP ' + str(sp_i + 1))
+                            label='EAP ' + str(sp_i + 1), lw=lw, alpha=alpha)
                 else:
-                    ax.plot(np.linspace(-(mea_pitch[0] - spacing) / 2., (mea_pitch[0] - spacing) / 2., signals.shape[2])
-                            + mea_pos[el, 0], np.transpose(sp[el, :]) + mea_pos[el, 1], lw=lw, color=colors,
-                            label='EAP ' + str(sp_i + 1))
+                    ax.plot(np.linspace(-(mea_pitch[0] - spacing) / 2., (mea_pitch[0] - spacing) / 2.,
+                                        signals.shape[2])
+                            + mea_pos[el, 0], np.transpose(sp[el, :]) + mea_pos[el, 1], color=colors[0],
+                            label='EAP ' + str(sp_i + 1), lw=lw, alpha=alpha)
 
     else:
-        for el in range(number_electrodes):
+        if len(colors) == len(channels):
+            color_list = colors
+        else:
+            color_list = [colors[0]] * number_electrodes
+        for i, el in enumerate(channels):
             ax.plot(np.linspace(-(mea_pitch[0] - spacing) / 2., (mea_pitch[0] - spacing) / 2., signals.shape[1]) +
                     mea_pos[el, 0], signals_norm[el, :] +
-                    mea_pos[el, 1], color=colors, lw=lw)
+                    mea_pos[el, 1], color=color_list[i], lw=lw, alpha=alpha)
 
     if hide_axis:
         ax.set_xticks([])
@@ -616,6 +652,7 @@ def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
 
     ax.set_xlim(
         [np.min(mea_pos[:, 0]) - (mea_pitch[0] - spacing) / 2., np.max(mea_pos[:, 0]) + (mea_pitch[0] - spacing) / 2.])
+    ax.set(**ax_kwargs)
 
     if scalebar:
         if time is None:
@@ -632,9 +669,9 @@ def plot_mea_recording(signals, mea, colors=None, lw=1, ax=None, spacing=None,
 
             ax.plot([pos_h[0], pos_h[0]], [pos_h[1], pos_h[1] + length_h], color='k', lw=2)
             if vscale is None:
-                ax.text(pos_h[0] + shift, pos_h[1] + length_h / 2., str(int(signalmin // 10 * 10)) + ' $\mu$V')
+                ax.text(pos_h[0] + shift, pos_h[1] + length_h / 2., str(int(signalmin // 10 * 10)) + ' uV')
             else:
-                ax.text(pos_h[0] + shift, pos_h[1] + length_h / 2., str(int(vscale)) + ' $\mu$V')
+                ax.text(pos_h[0] + shift, pos_h[1] + length_h / 2., str(int(vscale)) + ' uV')
             ax.plot([pos_w[0], pos_w[0] + length_w], [pos_w[1], pos_w[1]], color='k', lw=2)
             ax.text(pos_w[0] + shift, pos_w[1] - length_h / 3., str(time / 5) + ' ms')
 
@@ -687,6 +724,7 @@ def play_mea_recording(signals, mea, fs, window=1, step=0.1, colors=None, lw=1,
         The output animation
 
     '''
+    import matplotlib.pylab as plt
     import matplotlib.animation as animation
 
     n_window = int(fs * window)
@@ -707,7 +745,7 @@ def play_mea_recording(signals, mea, fs, window=1, step=0.1, colors=None, lw=1,
 
     # normalize to min peak
     if vscale is None:
-        signalmin = 1.5 * np.max(np.abs(signals))
+        signalmin = 1.5 * np.nanmax(np.abs(signals))
         signals_norm = signals / signalmin * mea_pitch[1]
     else:
         signals_norm = signals / vscale * mea_pitch[1]
@@ -789,6 +827,8 @@ def plot_cylinder_3d(bottom, direction, length, radius, color='k', alpha=.5, ax=
     ax: matplotlib axis
         The output axis
     '''
+    import matplotlib.pylab as plt
+
     if ax is None:
         fig = plt.figure()
         ax = fig.add_subplot(1, 1, 1, projection='3d')
@@ -827,6 +867,9 @@ def _make_3d_ellipse_patch(size, axis_1, axis_2, position, ax, facecolor='orange
     '''
     Helper function to make 3d ellipse patch.
     '''
+    import matplotlib.patches as patches
+    from mpl_toolkits.mplot3d import art3d
+
     p = patches.Circle((0, 0), size, facecolor=facecolor, alpha=alpha)
     ax.add_patch(p)
 
@@ -927,6 +970,9 @@ def _get_polygons_for_cylinder(pos_start, direction, length, radius, n_points, f
     '''
     Helper function to construct polygons from cylinders.
     '''
+    from matplotlib import colors as mpl_colors
+    from mpl_toolkits.mplot3d.art3d import Poly3DCollection
+
     x, y, z = _cylinder(pos_start,
                         direction,
                         length,
